<?php	/* lib_gps.php
         * (c) 2007, muevo Information Management & IT Consulting
         * author:	Andreas Forstinger (andreas.forstinger@muevo.at)
         * email:	office@muevo.at
         * web:		http://www.muevo.at
         */
        // library for gps related functions


/**
 * This function is derived from gps/gpsupdate.php by Sandra Schadenbauer
 * It is adapted so it can be used with Semacode recognition
 * @param float $lat latitude of GPS coordinate
 * @param float $lng longitude of GPS coordinate
 * @param bool  $ignoreThreshold indicates if the threshold (time between requests, 2min) should be ignored
 */
function momo_gps_get_locationbased_activities($lat, $lng, $ignoreThreshold = false){
    global $CFG, $USER;

    // write gps info to db
    $gpsTrack = new object();
    $gpsTrack->userid = $USER->id;
    $gpsTrack->lat = $lat;
    $gpsTrack->lng = $lng;
    $gpsTrack->timestamp = time();

    //Kurse, in denen der User Student ist, aus der DB lesen
    $courses=array();
    $myCourses=get_my_courses($USER->id);

    foreach($myCourses as $myCourse)
    {
        $courses[]=$myCourse->id;
    }
    /// check if location based data is available

    $gpsareas = get_records_select('gpsloc',"game=0");

    //Area von GPSTrack Objekt mit 0 initalisieren (falls der Punkt in keinem Polygon liegt steht in der DB '0')
    $gpsTrack->area=0;

    //wenn GPSLoc Objekte in der DB vorhanden sind
    if($gpsareas)
    {
        foreach ($gpsareas as $area)
        {
            //ist der GPS Bereich in einem Kurs, in dem der User Student ist?
            $inCourse=false;
            for($i=0; $i<count($courses); $i++)
            {
                if($courses[$i]==$area->course)
                    $inCourse=true;
            }

            if($inCourse==true)
            {
                //Lat und Lng Koordinaten werden in 2 Arrays aufgeteilt
                $xPoints=array();
                $yPoints=array();
                $anzPoints=0;
                $points=$header=explode("|",$area->gpspoints);

                foreach($points as $point)
                {
                    $lnglat=explode(",",$point);
                    $xPoints[]=$lnglat[1];
                    $yPoints[]=$lnglat[0];
                    $anzPoints++;
                }

                //Algorithmus um herauszufinden ob sich ein Punkt in einem Polygon befindet
                $j=0;
                $oddNotes=false;

                // check if the given point is the semacode (could be slightly outside the polygon)
                if( ($gpsTrack->lng .',' . $gpsTrack->lat) == $area->sema){
                    $oddNotes = true;
                }
                else{ // check all the polygon points
                    for ($i = 0; $i < $anzPoints; $i++)
                    {
                        $j++;
                        if($j==$anzPoints)
                            $j=0;
                        if(($yPoints[$i]<$lng && $yPoints[$j]>=$lng)||($yPoints[$j]<$lng && $yPoints[$i]>=$lng))
                        {
                            $result=bcadd($xPoints[$i],bcmul(bcdiv(bcsub($lng,$yPoints[$i],8),bcsub($yPoints[$j],$yPoints[$i],8),8),bcsub($xPoints[$j],$xPoints[$i],8),8),8);
                            if($result<$lat)
                                $oddNotes=!$oddNotes;
                        }
                    }
                }

                //Wenn sich der Punkt im Polygon befindet...
                if($oddNotes==true)
                {
                    //momo_log(date("d.m.y - H:i:s",time()).": Bereich=".$area->name,$action='gps debug',$url='');

                    //GPS Track Objekt mit GPS Bereichsnummer bef�llen
                    $gpsTrack->area=$area->id;


                    //Variable die bestimmt ob die Informationen zum GPS Bereich angezeigt werden
                    $showArea=false;

                    //letzte Position aus DB lesen
                    $lastposition = get_records_select('gpstrack',"userid = $USER->id", 'timestamp DESC', '*', 0, 1);
                    if($lastposition)
                    {
                        foreach($lastposition as $position)
                        {
                            //Befindet sich die letzte Position im selben Polygon wie die aktuelle?
                            if($position->area==$area->id)
                            {
                                //letzte GPS Bereichsver�nderung aus der DB lesen
                                $lastareas = get_records_select('gpschange',"userid=$USER->id",'timestamp DESC','*',0,1);
                                if($lastareas)
                                {
                                    foreach($lastareas as $lastarea)
                                    {
                                        //aktuelle Zeit bestimmen und mit der Zeit der letzten GPS Bereichver�nderung vergleichen
                                        //wenn seit der letzten Bereichver�nderung mehr als 2min vergangen sind wird die Information angezeigt
                                        //und ein neues GPSBereichver�nderungsobjekt in die DB geschrieben
                                        $curTime=time();
                                        $difference=bcsub($curTime,$lastarea->timestamp,0);
                                        //momo_log(date("d.m.y - H:i:s",time()).": ".$difference." sec",$action='gps debug',$url='');
                                        if($difference>=120 || $ignoreThreshold)
                                        {
                                            //momo_log(date("d.m.y - H:i:s",time()).": ".$difference." >=120",$action='gps debug',$url='');
                                            $gpsChange=new object();
                                            $gpsChange->userid=$USER->id;
                                            $gpsChange->timestamp=time();
                                            $gpsChange->area=$area->id;
                                            insert_record('gpschange',$gpsChange);
                                            $showArea=true;
                                        }
                                    }
                                }
                                //falls es noch kein GPS Bereichsver�nderungsobjekt in der DB gibt
                                else
                                {
                                    //momo_log('kein gps change vorhanden',$action='gps debug',$url='');
                                    $gpsChange=new object();
                                    $gpsChange->userid=$USER->id;
                                    $gpsChange->timestamp=time();
                                    $gpsChange->area=$area->id;
                                    insert_record('gpschange',$gpsChange);
                                    $showArea=true;
                                }
                            }
                            //wenn sich die aktuelle Position in einem anderen GPS Bereich befindet als die letzte bzw. wenn die letzte Position in keinem GPS Bereich war
                            else
                            {
                                //momo_log('neuer gps bereich',$action='gps debug',$url='');

                                $gpsChange=new object();
                                $gpsChange->userid=$USER->id;
                                $gpsChange->timestamp=time();
                                $gpsChange->area=$area->id;
                                insert_record('gpschange',$gpsChange);
                                $showArea=true;
                            }
                        }
                    }
                    //falls keine letzte Position vorhanden ist
                    else
                    {
                        //momo_log('keine lastposition',$action='gps debug',$url='');
                        $gpsChange=new object();
                        $gpsChange->userid=$USER->id;
                        $gpsChange->timestamp=time();
                        $gpsChange->area=$area->id;
                        insert_record('gpschange',$gpsChange);
                        $showArea=true;
                    }

                    //wenn die Informationen des GPS Bereich angezeigt werden sollen
                    if($showArea==true)
                    {
                        //momo_log(date("d.m.y - H:i:s",time()).": showArea=true",$action='gps debug',$url='');
                        $gpslocMod=0;

                        //ID des Moduls mit dem namen 'gpsloc' aus DB lesen
                        $modules=get_records_sql("select id from ".$CFG->prefix."modules where name='gpsloc'");
                        foreach($modules as $module)
                        {
                            $gpslocMod=$module->id;
                        }

                        //Section in der sich der aktuelle GPS Bereich befindet aus DB lesen
                        //(= Section in der ein Objekt mit der GPS Loc Modulnummer und der ID des GPS Bereichs befindet)
                        $sections=get_records_sql("select section from ".$CFG->prefix."course_modules where module=".$gpslocMod." and instance=".$area->id);
                        $curSection=0;
                        foreach($sections as $section)
                        {
                            $curSection=$section->section;
                        }

                        //�berpr�fen ob sich weitere Aktivit�ten in der Section befinden
                        //wenn ja, werden die Informationen angezeigt
                        $activities=get_records_sql("select id,module from ".$CFG->prefix."course_modules where section=".$curSection." and instance<>".$area->id);
                        if($activities)
                        {
                            //Session mit den ben�tigten Werten
                            $_SESSION["Section"]=$curSection;
                            $_SESSION["Area"]=$area->id;
                            $_SESSION["Name"]=$area->name;
                            $_SESSION["Description"]=$area->description;

                            // link generieren
                            $link = '<menu type="' . $CFG->momo_mle_server_link . '" u="gps/gpsactivities.php" dt="l" di="' . momo_get_icon_path('internet') . '" w="' . momo_truncate($area->name) .'" />';
                        }
                        else{
                            $link = false;
                        }
                    }
                    $gpsTrack->timestamp=time();
                    insert_record('gpstrack',$gpsTrack);
                }
            }
        }
    }

    /// write to db
    if($gpsTrack->area==0)
    {
        insert_record('gpstrack',$gpsTrack);
    }

    return $link;

}

?>