<?php	/* /momo/lib/lib_main.php, (c) 2008, Andreas Forstinger, www.mobilemoodle.org */
        // This is the main library containing functions
        // needed for the MOMO extension to run

//// ##########################################################################
////  basic request handling
//// ##########################################################################

/**
 * This function handles a request from MLE
 * @param XML file requested
 */
function momo_handle_request_xml($param) {
    global $CFG;

    // clear all output that eventually happened until now
    ob_clean();

    // if no specific file is requested show welcome page
    if (empty ($param) || $param == "index" || $param == "index.xml") {
        //        include ($CFG->dirroot . '/momo/pages/home.php');
        momo_print_error('error_file_not_found');
    }

    // upload tag workaround:
//     check if the url is malformed (trailing "?" without any parameters following)
        if(substr($param, -1) == '?'){
            // chop the character
            $param = substr($param,0,-1);
        }

    if (file_exists($CFG->dirroot . '/momo/pages/' . $param)) {
        if (momo_in_debug_mode())
            momo_log('page requested' . $CFG->dirroot . '/momo/pages/' . $param);
        include ($CFG->dirroot . '/momo/pages/' . $param);
    }

    // if requested file does not exist show error page
    else {
        if (momo_in_debug_mode())
            momo_log('page request failed ' . $param);
        momo_print_error('error_file_not_found');
    }

    // flush the page to client
    momo_flushXMLToClient();
}

/**
 * This function handles a content request from MLE
 * @param content requested
 */
function momo_handle_request_content($param) {
    global $CFG;

    // at the moment only MLO content can be downloaded
    require ($CFG->dirroot . '/mod/mlo/lib.php'); // include MLO module library

    // call function to get file path
    $file = mlo_get_file($param);

    // start output buffering
    ob_start();
    if ($file != false) {
        // open file handle
        $handle = fopen($file, rb);

        // set necessary headers
        header('Content-Type: application/zip', true);
        header('Content-Length: ' . filesize($file), true);

        // pass through file
        fpassthru($handle);

        // close file handle
        fclose($handle);
    } else {
        // send header with content length = 0
        header('Content-Type: application/zip', true);
        header('Content-Length: 0', true);
        echo ('error');
    }
    // flush buffer
    ob_end_flush();
}

/**
 * This function handles a bin request from MLE
 * @param binary requested
 */
function momo_handle_request_binary($param) {
    global $CFG;
    // clear output buffer
    @ ob_start();
    $success = false;
    $path = $param;

/// check what kind of binary is requested
    // a file in images directory was requested
    if(stripos($param,'images/')!== false){
        if(file_exists($CFG->dirroot . '/momo/' . $param)){
            $path = $CFG->dirroot . '/momo/' . $param;
            $success = true;
        }
    }
    // an user image has been requested
    else if (stripos($param,'user')!== false){
        $userid = optional_param('id',null);
        $file = optional_param('file',null);
        if (file_exists($CFG->dataroot . '/users/' . $userid . '/' . $file)){
            $path = $CFG->dataroot . '/users/' . $userid . '/' . $file;
            $success = true;
        }
    }

    // a file from data directory was requested
    else if(file_exists($CFG->dataroot . $param)){
        $path = $CFG->dataroot . $param;
        $success = true;
    }


    if ($success) {
        if (momo_in_debug_mode())
            momo_log('binary requested: ' . $path);
        momo_flushBinaryToClient($path);

    } else {
        if (momo_in_debug_mode())
            momo_log('binary request failed: ' . $path);
        momo_print_error('error_file_not_found');
    }

}

/**
 * This function adds the necessary headers and flushes content currently contained in #
 * output buffer to mobile client
 * @uses output buffer
 */
function momo_flushXMLToClient() {
    // get content from buffer
    $content = ob_get_clean();

    // encode everything to utf-8
    $content = mb_convert_encoding($content,'UTF-8','auto');

    // set content length
    header('Content-Length: ' . (string) strlen($content), true);
    // set content type
    header('Content-Type: text/xml; charset=UTF-8',true);
    // stop output buffering
    @ob_end_flush();
    echo $content;
    // stop execution
    exit;
}

/**
 * This function adds the necessary headers and flushes the given file to mobile client
 * @param $binary path to binary file to be flushed
 */
function momo_flushBinaryToClient($binary){
        // clean output buffer
        @ob_clean();
        // set 'Content-Length' Header as it is required by MLE
        header('Content-Length: ' . (string) filesize($binary),true);
        header('Content-Type: application/octetstream',true);
        // open file in readonly, binary mode
        $file = fopen($binary, 'rb');
        // read file and close
        fpassthru($file);
        fclose($file);
        @ob_end_flush();
        exit;
}

//// ##########################################################################
////  general helper functions
//// ##########################################################################

/**
 * This method writes the given message to internal moodle log. This is a simple shortcut to
 * Moodle "add_to_log()" function, which sets user, course and module.
 */
function momo_log($message, $action = 'debug', $url = '') {
    global $USER, $COURSE;
    // add the given message to moodle log
    if (isset ($USER->id))
        add_to_log($COURSE->id, 'momo', $action, $url, $message, '', $USER->id);
    else
        add_to_log($COURSE->id, 'momo', $action, $url, $message, '', '');
}

/**
 * This function simply returns true if momo is running in debug mode.
 */
function momo_in_debug_mode() {
    global $CFG;
    return (!empty($CFG->momo_debug_mode)) ? true : false;
}

/**
 * This function returns a file url compatible to MOMO requirements for requesting binaries from server.
 * @param $filepath - path to file relative to /moodledata/
 */
function momo_get_binary_file_url($filepath) {
    global $CFG;
    return '/'. $filepath;
}

/**
 * This function checks the given URL and if it is pointing inside this site, login information is added
 * @param $link - URL of the link to be encoded
 * @return String the given link including additional information (URL parameters)
 */
function momo_encode_login_information($link){
    global $CFG,$USER;
    // firstly check if given URL is pointing inside this site, otherwise we do not need to add login data
    $check = stripos($link, $CFG->wwwroot);
    if ($check !== false) { // URL is pointing inside this site so append login info
        $userinfo = 'momouser' . '=' . $USER->id;
        $passhash = 'momosecret' . '=' . md5($USER->username . $USER->id . $USER->password);
        // check if link already contains parameters ('?')
        $delimiter = (strpos($link,'?') === false) ? '?' : '&amp;';
        // build link
//        $link = 'file/' . $link . $delimiter . $userinfo . '&amp;' . $passhash;
        $link = $link . $delimiter . $userinfo . '&amp;' . $passhash;
    }
    return $link;
}

// define a backup function for "stripos" on systems < PHP5
if (!function_exists("stripos")) {
  function stripos($str,$needle,$offset=0)
  {
      return strpos(strtolower($str),strtolower($needle),$offset);
  }
}

//// ##########################################################################
////  Authentication and Security related functions
//// ##########################################################################

function momo_login() {
global $COURSE, $USER, $SESSION, $CFG;
    // check if user is already logged in
    if(!isloggedin()){

        // call course_setup so everything is set up correctly
        course_setup(0);

        // store referer
        if (!empty($_SERVER['HTTP_REFERER'])) {
            $SESSION->fromurl  = $_SERVER['HTTP_REFERER'];
        }

        // do authentication (uses configured moodle auth plugins)
        $username = strtolower(momo_basic_getUser());
        $password = momo_basic_getPassword();

        $user = authenticate_user_login($username,$password);

        if($user){
            $USER = $user;
            momo_log($USER->id,'login',"view.php?id=$USER->id&course=".SITEID);

            // set correct last login time etc.
            update_user_login_times();

            // load user capabilities
            load_all_capabilities();

            // check if user profile is correctly set up
            if(user_not_fully_set_up($USER)){
                 momo_print_error('error_account');
                 exit;
            }

            // set up a sesskey for the user
            sesskey();

        }
        else{ // authentication failed, show error page
            momo_log("login failed for $username",'login','index.php');
            error_log('[client ' . $_SERVER['REMOTE_ADDR'] . "]  $CFG->wwwroot  Failed Login:  $username  " . $_SERVER['HTTP_USER_AGENT']);
            momo_print_error('error_login');
            exit;
        }
    }
}

function momo_login_to_course($id){
    global $COURSE, $USER;

    // if not already logged in call login function
    if(!isloggedin()){
        momo_login();
    }

    // check if user is already logged in to this course
    if($COURSE->id != $id){
        // prepare course
        course_setup($id);

        // check if is user is allowed to access this course
        if(!$context = get_context_instance(CONTEXT_COURSE, $COURSE->id)){
            momo_print_error('error_security_problem');
            exit;
        }
        if(!has_capability('moodle/course:view', $context)){
            momo_print_error('error_security_course_not_allowed');
            exit;
        }
    }


}

//// ##########################################################################
////  MLE related functions (low level)
//// ##########################################################################

/**
 * This function decrypts a given string using a simple algorithm
 * @uses MLE_PASSKEY, MLE_PASSOPERATION
 * @param the encrypted string
 * @return null if operation fails, otherwise the decrypted string
 */
function momo_basic_decrypt($encrypted) {
    if (empty ($encrypted))
        return null;
    global $MLE_PASSKEY, $MLE_PASSKEYOPERATION;
    // prepare variables
    $keyCount = $operationCount = 0;
    $decrypted = "";
    // split string
    $tokens = explode(",", $encrypted);
    // iterate tokens
    foreach ($tokens as $token) {
        $code = $token;
        if ($MLE_PASSKEYOPERATION[$operationCount] == 1) {
            $code -= $MLE_PASSKEY[$keyCount];
        } else
            $code += $MLE_PASSKEY[$keyCount];
        // append character
        $decrypted .= chr($code);
        // update counters
        $keyCount++;
        $operationCount++;
        if ($keyCount >= count($MLE_PASSKEY))
            $keyCount = 0;
        if ($operationCount >= count($MLE_PASSKEYOPERATION))
            $operationCount = 0;
    }

    return $decrypted;
}

/**
 * This function encrypts a given string using a simple algorithm
 * @uses MLE_PASSKEY, MLE_PASSOPERATION
 * @param the string to be encrypted
 * @return null if operation fails, otherwise the encrypted string
 */
function momo_basic_encrypt($string) {
    // prepare variables
    global $MLE_PASSKEY, $MLE_PASSKEYOPERATION;
    $keyCount = $operationCount = 0;
    $encrypted = '';

    for ($i = 0; $i < strlen($string); $i++) {
        // append ","
        if ($i > 0)
            $encrypted .= ',';
        // get ascii value
        $code = ord($string {
            $i });
        // encode
        if ($MLE_PASSKEYOPERATION[$operationCount] == 1)
            $encrypted .= $code + $MLE_PASSKEY[$keyCount];
        else
            $encrypted .= $code - $MLE_PASSKEY[$keyCount];
        $keyCount++;
        if ($keyCount >= count($MLE_PASSKEY))
            $keyCount = 0;
        $operationCount++;
        if ($operationCount >= count($MLE_PASSKEYOPERATION))
            $operationCount = 0;
    }
    return $encrypted;
}

/**
* Returns a header value
* either from POST or from GET
* if header wasn't found, $defaultReturn is returned
*/
function momo_basic_get_header($headerName, $defaultReturn) {
    global $_SERVER;
    if (strlen($headerName) <= 0)
        return $defaultReturn;
    if (array_key_exists($headerName, $_SERVER))
        return $_SERVER[$headerName];
    else
        if (array_key_exists($headerName, $GLOBALS))
            return $GLOBALS[$headerName];
    $headerName = "HTTP_" . $headerName; // Apache + PHP add "HTTP_" to every var
    if (array_key_exists($headerName, $_SERVER))
        return $_SERVER[$headerName];
    else
        if (array_key_exists($headerName, $GLOBALS))
            return $GLOBALS[$headerName];
    return $defaultReturn;
}

/**
 * Checks the Request Header to check whether the current user is
 * accessing script via MOMO client or not
 * (usually you would check the USER_AGENT header, but there is a special header in MLE)
 * @return true if page is accessed via momo client or false otherwise
 */
function momo_basic_isMobileUser() {
	global $CFG;
    $agent = momo_basic_get_header($CFG->momo_mle_param_platform ,false);
    if (strncmp($agent, $CFG->momo_mle_platform, strlen($CFG->momo_mle_platform)) == 0)
        return true;
    else
        return false;
}

/**
 * This method returns the current users username
 * @return username or false on error
 */
function momo_basic_getUser(){
	global $CFG;
    // get user from MLE header value
    $username = momo_basic_get_header($CFG->momo_mle_param_user, null);
    if(empty($username))
        return false;
    else
        return $username;
}

/**
 * This method returns the current users password in plain text (i.e. already decrypted)
 * @return password or false on error
 */
function momo_basic_getPassword(){
	global $CFG;
    // get password from MLE header value
    $encpassword = momo_basic_get_header($CFG->momo_mle_param_password, null);
    if(empty($encpassword))
        return false;
    else{
        // decrypt the password
        $password = momo_basic_decrypt($encpassword);
        if(empty($password))
            return false;
        else
            return $password;
    }
}

?>